@@ -32,7 +32,7 @@ templates_dir = os.path.join(root, 'templates')
 fuzz_words_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'wfuzz/wordlist')
 fuzz_types = ["general", "injections", "vulns", "webservices", "stress", "others"]
 ANOMOLY_THRESHOLD = 1.07142
-RESERVED_PARAMETERS = ["$RESPONSE_ERROR"]
+RESERVED_PARAMETERS = ["$RESPONSE_ERROR","$CV_EVENT_ID"]
 
 
 def keypaths(nested):
@@ -87,7 +87,8 @@ class CloudvectorDAST(object):
         else:
             self.apispec_one_path = None
         self.apispec_two_path = APISpecTwo
-        self.openapispec_obj = OpenApiSpecDiff.OpenApiSpecDiff(self.apispec_one_path, self.apispec_two_path)
+        if ".txt" not in self.apispec_two_path:
+            self.openapispec_obj = OpenApiSpecDiff.OpenApiSpecDiff(self.apispec_one_path, self.apispec_two_path)
         self.ceobj = CommunityEdition.CommunityEdition(ce_host, ce_username, ce_password)
         print("\n\t\t\t\t\t\t\t\t\t----------------- DAST - For CloudVector APIShark events "
               "-----------------")
@@ -99,25 +100,30 @@ class CloudvectorDAST(object):
             if not os.path.exists((os.path.join(os.getcwd(),"wfuzz"))):
                 shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)),"wfuzz"), os.path.join(os.getcwd(),"wfuzz"))
         self.params_captured_in_traffic = {}
-        changed_apis = self._get_changed_apis()
-        apis_to_check = changed_apis["changed"]
-        apis_to_check.update(changed_apis["new"])
-        if self.apispec_one_path:
-            self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path, changed_apis)
+        self.prepped_spec = []
+        if ".txt" not in self.apispec_two_path:
+            changed_apis = self._get_changed_apis()
+            apis_to_check = changed_apis["changed"]
+            apis_to_check.update(changed_apis["new"])
+            if self.apispec_one_path:
+                self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path, changed_apis)
+            else:
+                self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path)
         else:
-            self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path)
+            with open(self.apispec_two_path) as fobj:
+                apis_to_check = {str(_).replace("\n",""):"" for _ in fobj.readlines()}
         self.input_json = {}
         self.assertions_map = {}
-        for file in input_params_file.split(";"):
-            print("loading variables from input file " + str(file) + " .....")
-            input_json = {}
-            if os.path.exists(file):
-                if ".json" in file:
-                    with open(file) as fobj:
-                        input_json = json.load(fobj)
-                else:
-                    input_json = self._load_input_from_files(file)
-            self.input_json.update(input_json)
+        # for file in input_params_file.split(";"):
+        #     print("loading variables from input file " + str(file) + " .....")
+        #     input_json = {}
+        #     if os.path.exists(file):
+        #         if ".json" in file:
+        #             with open(file) as fobj:
+        #                 input_json = json.load(fobj)
+        #         else:
+        #             input_json = self._load_input_from_files(file)
+        #     self.input_json.update(input_json)
 
 
 
@@ -125,11 +131,11 @@ class CloudvectorDAST(object):
         # if not cv_events:
         #     cv_events = self.prepped_spec
         cv_events += self.prepped_spec
-        self._process_input_json()
+        #self._process_input_json()
         if cover_only_diff == "y":
             self._process_param_diff(apis_to_check, True)
-        else:
-            self._process_param_diff(apis_to_check, False)
+        # else:
+        #     self._process_param_diff(apis_to_check, False)
 
         # params_captured_in_traffic_new = deepcopy(self.params_captured_in_traffic)
         # for k,v in self.params_captured_in_traffic.items():
@@ -151,7 +157,7 @@ class CloudvectorDAST(object):
 
         #self.params_captured_in_traffic = params_captured_in_traffic_new
         with open("tests/params_captured.json", "w+") as fobj:
-            json.dump(self.params_captured_in_traffic, fobj)
+            json.dump(self.params_captured_in_traffic, fobj, indent=4)
         self.create_pyfixtures()
         # self.create_fuzzfixtures()
         self.create_pytest_methods(cv_events, custom_validations)
@@ -191,6 +197,7 @@ class CloudvectorDAST(object):
 
     def _prepare_spec_for_test_generation(self, input_spec, specific_apis=None):
         parsed_new_spec = self._get_spec_parsed(input_spec)
+        #print(parsed_new_spec)
         apis_from_spec = []
         if "servers" in parsed_new_spec:
             from urllib.parse import urlparse
@@ -210,7 +217,9 @@ class CloudvectorDAST(object):
                     continue
             if api not in self.respcodes_from_spec:
                 self.respcodes_from_spec[api] = {}
+            #print("----------")
             for method, minfo in info.items():
+                #print(method, minfo)
                 x = {}
                 x["method"] = method.lower()
                 x["host"] = host
@@ -235,6 +244,7 @@ class CloudvectorDAST(object):
                 self.respcodes_from_spec[x["http-req-url"]][method] = list(minfo.get("responses").keys())
                 x["url"] = x["url"].replace("{", "{{").replace("}", "}}")
                 for _ in minfo.get("parameters", []):
+                    #print("\n\n"+str(_)+"\n\n")
                     if _.get("in") == "header":
                         x["header"][_.get("name")] = ""
                     elif _.get("in") == "query":
@@ -247,16 +257,24 @@ class CloudvectorDAST(object):
                         if _.get("name"):
                             x["params"].append(_.get("name"))
                             x["body"][_.get("name")] = ""
+                    #print("this....."+str(_.get("value")), api, method)
                     if _.get("value"):
                         if type(_.get("value")) is list:
-                            for e in _.get("value"):
-                                x["detailed_params"].update(e)
-                        else:
-                            x["detailed_params"].update(_.get("value"))
-                self.params_from_spec[api] = {_:[] for _ in x["params"]}
-                self.params_details_from_spec.update(x["detailed_params"])
+                            #print("its a list....")
+                            self.params_details_from_spec[str(_.get("name"))] = [x["name"] for x in _.get("value")]
+                        # else:
+                        #     print("not list...")
+                        #     x["detailed_params"].update(_.get("value"))
+                if api not in self.params_from_spec:
+                    self.params_from_spec[api] = {}
+                self.params_from_spec[api][method] = {_:[] for _ in x["params"]}
+                #.update(x["detailed_params"])
                 # if str(self.cv_config["authentication"]["api"]).lower() in str(x["url"]).lower():
                 #     self.cvdast_to_auth_info = x
+                #print("\n")
+                #print(self.params_details_from_spec)
+                #print("\n")
+                #print(self.params_from_spec)
                 apis_from_spec.append([x, None])
         return apis_from_spec
 
@@ -297,23 +315,23 @@ class CloudvectorDAST(object):
         return params_info
 
     def _get_spec_parsed(self, input_path):
-        return self.openapispec_obj.scan_input_spec(self.openapispec_obj.parse_spec(input_path))
+        return self.openapispec_obj.parse_spec(self.apispec_two_path)
 
     def _get_changed_apis(self):
         return self.openapispec_obj.diff
 
-    def _process_input_json(self):
-        for key, value in self.input_json.items():
-            if type(value) is list:
-                if "others" not in self.params_captured_in_traffic:
-                    self.params_captured_in_traffic["others"] = {}
-                self.params_captured_in_traffic["others"].update({key: value})
-            elif type(value) is dict:
-                for name, values in value.items():
-                    if "null" in values:
-                        values.remove("null")
-                    if key in self.params_captured_in_traffic:
-                        self.params_captured_in_traffic[key].update({name: values})
+    # def _process_input_json(self):
+    #     for key, value in self.input_json.items():
+    #         if type(value) is list:
+    #             if "others" not in self.params_captured_in_traffic:
+    #                 self.params_captured_in_traffic["others"] = {}
+    #             self.params_captured_in_traffic["others"].update({key: value})
+    #         elif type(value) is dict:
+    #             for name, values in value.items():
+    #                 if "null" in values:
+    #                     values.remove("null")
+    #                 if key in self.params_captured_in_traffic:
+    #                     self.params_captured_in_traffic[key].update({name: values})
 
     def _process_param_diff(self, changed_apis, only_diff):
         if only_diff:
@@ -322,31 +340,44 @@ class CloudvectorDAST(object):
         for api, info in changed_apis.items():
             # if only_diff:
             #     self.params_captured_in_traffic[api] = {}
+            # for method, params in info.items():
+            #     for param in params:
+            #         if api in self.params_captured_in_traffic:
+            #             if param.get("name") not in self.params_captured_in_traffic[api]:
+            #                 self.params_captured_in_traffic[api].update({param.get("name"): []})
+            iflag = True
             for method, params in info.items():
+                if only_diff and iflag:
+                    self.params_captured_in_traffic = {api:{"200":{str(method).lower():[]}}}
+                    iflag = False
+                all_params = {}
                 for param in params:
                     if api in self.params_captured_in_traffic:
-                        if param.get("name") not in self.params_captured_in_traffic[api]:
-                            self.params_captured_in_traffic[api].update({param.get("name"): []})
-
-        for api, info in self.params_captured_in_traffic.items():
-            for param, value in info.items():
-                if param in self.input_json:
-                    self.params_captured_in_traffic[api][param] = self.input_json[param]
-                if api in self.input_json:
-                    if param in self.input_json[api]:
-                        self.params_captured_in_traffic[api][param] = self.input_json[api][param]
+                        if only_diff and iflag:
+                            #self.params_captured_in_traffic[api]["200"][str(method).lower()] = []
+                            all_params[param.get("name")] = ""
+                self.params_captured_in_traffic[api]["200"][str(method).lower()].append(all_params)
+        # for api, info in self.params_captured_in_traffic.items():
+        #     for param, value in info.items():
+        #         if param in self.input_json:
+        #             self.params_captured_in_traffic[api][param] = self.input_json[param]
+        #         if api in self.input_json:
+        #             if param in self.input_json[api]:
+        #                 self.params_captured_in_traffic[api][param] = self.input_json[api][param]
 
     def _get_fuzz_details(self, fuzz_type):
         fuzz_values = []
-        if os.path.isdir(os.path.join(fuzz_words_dir, fuzz_type)):
-            for (root, dirs, files) in os.walk(os.path.join(fuzz_words_dir, fuzz_type), topdown=True):
-                for _file in files:
-                    fuzz_values.append({"file": os.path.join(os.path.join("wfuzz/wordlist/", fuzz_type), _file),
-                                        "fuzz_type": fuzz_type})
+        # if os.path.isdir(os.path.join(fuzz_words_dir, fuzz_type)):
+        #     for (root, dirs, files) in os.walk(os.path.join(fuzz_words_dir, fuzz_type), topdown=True):
+        #         for _file in files:
+        #             fuzz_values.append({"path": os.path.join("wfuzz/wordlist/", fuzz_type),
+        #                                 "fuzz_type": fuzz_type})
                     # values = []
                     # with open(file) as _file:
                     #     values = _file.readlines()
                     # fuzz_values[file] = values
+        fuzz_values.append({"path": os.path.join("wfuzz/wordlist/", fuzz_type),
+                                        "fuzz_type": fuzz_type})
         return fuzz_values
 
     def _get_hosts_input(self, hosts):
@@ -389,40 +420,45 @@ class CloudvectorDAST(object):
                 auth_prefix = ""
             elif auth_type == "bearer":
                 auth_prefix = "Bearer"
-        for api,params_info in self.params_from_spec.items():
-            for statuscode in self.params_captured_in_traffic:
-                if int(statuscode) not in range(200,299):
-                    if not self.params_captured_in_traffic.get(statuscode):
-                        continue
-                    if api not in self.params_captured_in_traffic[statuscode]:
-                        self.params_captured_in_traffic[statuscode][api] = {}
-                    for k, v in params_info.items():
-                        if k not in self.params_captured_in_traffic[statuscode].get(api,{}):
-                            self.params_captured_in_traffic[statuscode][api][k]=v
+            else:
+                auth_prefix = auth_type
+        # for api,params_info in self.params_from_spec.items():
+        #     for statuscode in self.params_captured_in_traffic:
+        #         if int(statuscode) not in range(200,299):
+        #             if not self.params_captured_in_traffic.get(statuscode):
+        #                 continue
+        #             if api not in self.params_captured_in_traffic[statuscode]:
+        #                 self.params_captured_in_traffic[statuscode][api] = {}
+        #             for k, v in params_info.items():
+        #                 if k not in self.params_captured_in_traffic[statuscode].get(api,{}):
+        #                     print(self.params_captured_in_traffic[statuscode][api])
+        #                     self.params_captured_in_traffic[statuscode][api][k]=v
         nested_params={}
         more_params = []
+        #print("???"+str(self.params_details_from_spec))
         for param, info in self.params_details_from_spec.items():
-            tmp = []
-            if type(info) is list:
-                info = info[0]
-            for k, v in self._recursive_items(info):
-                if type(v) is not dict:
-                    tmp.append(k)
-            nested_params[param] = tmp
-            more_params+=tmp
+            #tmp = []
+            # if type(info) is list:
+            #     info = info[0]
+            # for k, v in self._recursive_items(info):
+            #     if type(v) is not dict:
+            #         tmp.append(k)
+            nested_params[param] = info
+            more_params+=info
+        #print("\n\n"+str(nested_params)+"\n\n")
 
         params_for_fixtures = []
-        for k, v in self.params_captured_in_traffic.items():
-            for api, plist in v.items():
-                #print(api, plist)
-                for each in plist:
-                    if type(each) is str:
-                        if each not in params_for_fixtures and each not in RESERVED_PARAMETERS:
-                            params_for_fixtures.append(each)
-                    elif type(each) is dict:
-                        for _ in each.keys():
-                            if _ not in params_for_fixtures and _ not in RESERVED_PARAMETERS:
-                                params_for_fixtures.append(_)
+        for api,info in self.params_captured_in_traffic.items():
+            for rspcode, minfo in info.items():
+                for method, plist in minfo.items():
+                    for each in plist:
+                        if type(each) is str:
+                            if each not in params_for_fixtures and each not in RESERVED_PARAMETERS:
+                                params_for_fixtures.append(each)
+                        elif type(each) is dict:
+                            for _ in each.keys():
+                                if _ not in params_for_fixtures and _ not in RESERVED_PARAMETERS:
+                                    params_for_fixtures.append(_)
 
 
         with open("tests/conftest.py", 'w+') as fh:
@@ -463,7 +499,7 @@ class CloudvectorDAST(object):
     def _create_assertions_map(self, url, params, req_payload, resp_payload):
         assertions_map = {}
         if url not in self.assertions_map:
-            url = url.replace("/", "_").replace("-", "_").replace("{", "").replace("}", "").split("?")[0]
+            url = url.replace("//","/").replace("/", "_").replace("-", "_").replace("{", "").replace("}", "").split("?")[0]
             self.assertions_map[url] = {}
         if None in params:
             params.remove(None)
@@ -504,8 +540,12 @@ class CloudvectorDAST(object):
         apis_to_be_tested = {}
         files_created = set()
         print("\n\ncreating Pytest test methods....\n")
-        new_spec_info = self._scan_input_spec(self.apispec_two_path)
-        hosts = new_spec_info.get("servers", {}).get("url", [])
+        try:
+            new_spec_info = self._scan_input_spec(self.apispec_two_path)
+            hosts = new_spec_info.get("servers", {}).get("url", [])
+        except AttributeError:
+            new_spec_info = {}
+            hosts = []
         #print(cv_events)
         for _ in cv_events:
             #print(_)
@@ -514,9 +554,9 @@ class CloudvectorDAST(object):
                 if int(_[1]["attributes"]["http_rsp_status_code"]) not in range(200, 299):
                     continue
             if event["url"]:
-                api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
-                    0]
-
+                # api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+                #     0]
+                api = str(event["http-req-url"]).lstrip("/").replace("/", "_").replace("-", "_").split("?")[0]
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
                 #apis_to_be_tested[api]["method"] = event["method"]
@@ -535,7 +575,6 @@ class CloudvectorDAST(object):
                     known_params = apis_to_be_tested[api][method].get("params")
                 else:
                     known_params = []
-
                 apis_to_be_tested[api][method]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"),{}).get(method,[])
 
                 params_in_traffic = event.get("body",{}).keys()
@@ -559,7 +598,7 @@ class CloudvectorDAST(object):
         with open(os.path.join(templates_dir, 'test_api.j2')) as file_:
             template = Template(file_.read())
         #host_url = [list(set([_ for _ in hosts if _ != '']))[-1]]
-
+        # print(apis_to_be_tested)
         for k, v in apis_to_be_tested.items():
             # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
             filename = str(k).replace("/", "_").replace("{", "").replace("}", "")
@@ -576,9 +615,10 @@ class CloudvectorDAST(object):
                 status_codes_for_api = self.respcodes_from_spec.get(api_name,{})
             else:
                 status_codes_for_api = {}
+
             with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
                 fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
-                                         custom_validations=extra_validations, STATUS_CODES=status_codes_for_api))
+                                         custom_validations=extra_validations, STATUS_CODES=status_codes_for_api, TEST_MANAGEMENT=self.cv_config.get("upload_result_to")))
             files_created.add("test_" + str(filename))
         print("\n\t\t......done creating pytest methods: " + str(files_created))
 
@@ -586,8 +626,12 @@ class CloudvectorDAST(object):
         apis_to_be_tested = {}
         files_created = set()
         print("\n\ncreating Pytest test methods for fuzzing....\n")
-        new_spec_info = self._scan_input_spec(self.apispec_two_path)
-        hosts = new_spec_info.get("servers", {}).get("url", [])
+        try:
+            new_spec_info = self._scan_input_spec(self.apispec_two_path)
+            hosts = new_spec_info.get("servers", {}).get("url", [])
+        except AttributeError:
+            new_spec_info = {}
+            hosts = []
         # for _ in cv_events:
         #     event = _[0]
         #     if _[1]:
@@ -622,7 +666,7 @@ class CloudvectorDAST(object):
                 if _[1]["attributes"]["http_rsp_status_code"] not in ["200", "201"]:
                     continue
             if event["url"]:
-                api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+                api = str(event["http-req-url"]).lstrip("/").replace("/", "_").replace("-", "_").split("?")[
                     0]
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
@@ -774,6 +818,7 @@ class CloudvectorDAST(object):
             request["header"] = header
             request["body"] = req_params_found
             request["rsp_body"] = rsp_params_found
+            request["method"] = str(event["attributes"].get("http_method")).lower()
             #print(event["attributes"]["event_json"])
             request["rsp_code"] = str(event["attributes"]["http_rsp_status_code"]).split(" ")[0]
             if request["rsp_code"] not in [200, 201]:
@@ -798,18 +843,27 @@ class CloudvectorDAST(object):
             #     except TypeError:
             #         pass
 
-            if str(request["rsp_code"]) not in self.params_captured_in_traffic:
-                self.params_captured_in_traffic[request["rsp_code"]] = {}
-            if str(event["attributes"]["http_path"]) not in self.params_captured_in_traffic[request["rsp_code"]]:
-                self.params_captured_in_traffic[request["rsp_code"]].update( { str(event["attributes"]["http_path"]) : [] } )
-
+            # if str(request["rsp_code"]) not in self.params_captured_in_traffic:
+            #     self.params_captured_in_traffic[request["rsp_code"]] = {}
+            # if str(event["attributes"]["http_path"]) not in self.params_captured_in_traffic[request["rsp_code"]]:
+            #     self.params_captured_in_traffic[request["rsp_code"]].update( { str(event["attributes"]["http_path"]) : [] } )
+            url = event["attributes"]["http_path"]
+            # if url.endswith("/"):
+            #     url = url [:-1]
+            if str(url) not in self.params_captured_in_traffic:
+                self.params_captured_in_traffic[url] = {}
+            if str(request["rsp_code"]) not in self.params_captured_in_traffic[url]:
+                self.params_captured_in_traffic[url].update( { str(request["rsp_code"]) : {} } )
+            if str(request["method"]) not in self.params_captured_in_traffic[url][request["rsp_code"]]:
+                self.params_captured_in_traffic[url][request["rsp_code"]].update( { str(request["method"]) : [] } )
             params_found = {}
             for param, value in req_params_found.items():
                 params_found[param] = value
             if params_found:
                 if request["rsp_code"] not in [200, 201]:
                     params_found["$RESPONSE_ERROR"] = request.get("rsp_error")
-                self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])].append(params_found)
+                    params_found["$CV_EVENT_ID"] = "CV-EVENT-"+str(event["id"])
+                self.params_captured_in_traffic[str(url)][request["rsp_code"]][request["method"]].append(params_found)
                 # if param in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])]:
                 #     # self.params_captured_in_traffic[str(event["attributes"]["http_path"]).lower()] = {}
                 #     #if value not in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][
